'use strict';

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/**
 *  cxs-like class
 *
 *  const createStyle = new CSS({
 *    id: 'style-element-id',
 *    prefix: 'dxs',
 *    styleElement: document.getElementById('custom-style-tag'),
 *  })
 */

var CSS = function () {
  function CSS() {
    var _this = this;

    var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    _classCallCheck(this, CSS);

    var _options$prefix = options.prefix,
        prefix = _options$prefix === undefined ? '_dxs' : _options$prefix,
        _options$id = options.id,
        id = _options$id === undefined ? '__DXS__' : _options$id,
        styleElement = options.styleElement;


    this.cache = {};
    this.id = id;
    this.prefix = prefix;
    this.rules = [];
    this.insert = function (rule) {
      return rules.push(rule);
    };

    this.css = function () {
      return _this.rules.sort().join('');
    };

    this.reset = function () {
      _this.cache = {};
      _this.rules = [];
    };

    if (typeof document !== 'undefined') {
      this.styleElement = createStyleElement({ id: id, styleElement: styleElement });
      this.sheet = this.styleElement.sheet;

      this.insert = function (rule) {
        _this.rules.push(rule);
        _this.sheet.insertRule(rule, _this.sheet.cssRules.length);
      };
    }

    return this.createStyle.bind(this);
  }

  _createClass(CSS, [{
    key: 'parseRules',
    value: function parseRules(obj) {
      var child = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : '';
      var media = arguments[2];

      var rules = [];
      var declarations = [];

      for (var key in obj) {
        var value = obj[key];

        if (value === null) continue;

        if ((typeof value === 'undefined' ? 'undefined' : _typeof(value)) === 'object') {
          var _media = /^@/.test(key) ? key : null;
          var _child = _media ? child : child + noAnd(key);
          this.parseRules(value, _child, _media).forEach(function (r) {
            return rules.push(r);
          });
          continue;
        }

        var dec = createDeclaration(key, value);
        declarations.push(dec);
      }

      rules.unshift({
        media: media,
        child: child,
        declarations: declarations
      });

      return rules;
    }
  }, {
    key: 'parse',
    value: function parse(obj) {
      var _this2 = this;

      var rules = this.parseRules(obj);
      var classNames = [];

      rules.forEach(function (rule) {
        var cacheKey = JSON.stringify(rule);
        if (_this2.cache[cacheKey]) {
          classNames.push(_this2.cache[cacheKey]);
          return;
        }
        var className = _this2.prefix + _this2.rules.length.toString(36);
        classNames.push(className);
        var ruleset = createRule(Object.assign(rule, { className: className }));
        _this2.insert(ruleset);
        _this2.cache[cacheKey] = className;
      });

      return classNames.join(' ');
    }
  }, {
    key: 'createStyle',
    value: function createStyle() {
      var _this3 = this;

      for (var _len = arguments.length, styles = Array(_len), _key = 0; _key < _len; _key++) {
        styles[_key] = arguments[_key];
      }

      return styles.map(function (style) {
        return _this3.parse(style);
      }).join(' ').trim();
    }
  }]);

  return CSS;
}();

var createStyleElement = function createStyleElement(_ref) {
  var id = _ref.id,
      styleElement = _ref.styleElement;

  if (styleElement) return styleElement;
  var style = document.getElementById(id) || document.head.appendChild(document.createElement('style'));
  style.id = id;
  return style;
};

var hyph = function hyph(s) {
  return s.replace(/[A-Z]|^ms/g, '-$&').toLowerCase();
};
var mx = function mx(rule, media) {
  return media ? media + '{' + rule + '}' : rule;
};
var noAnd = function noAnd(s) {
  return s.replace(/&/g, '');
};
var createDeclaration = function createDeclaration(key, value) {
  return hyph(key) + ':' + value;
};
var createRule = function createRule(_ref2) {
  var className = _ref2.className,
      child = _ref2.child,
      media = _ref2.media,
      declarations = _ref2.declarations;
  return mx('.' + (className + child) + '{' + declarations.join(';') + '}', media);
};

module.exports = CSS;