'use strict';

var _require = require('prettier'),
    format = _require.format;

var templates = require('./templates');
var composite = require('./templates/composite');

var _require2 = require('./templates/util'),
    stringifyObject = _require2.stringifyObject,
    getExtensionImport = _require2.getExtensionImport,
    getComp = _require2.getComp,
    getSystemFuncs = _require2.getSystemFuncs;

var parse = function parse(conf) {
  return Object.assign({}, conf, {
    systemFunctions: getSystemFuncs(conf.system || []),
    extensionImport: getExtensionImport(conf.type),
    type: getComp(conf.type || 'div'),
    style: stringifyObject(conf.style || {}),
    props: stringifyObject(conf.props || {})
  });
};

module.exports = function (config) {
  var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  var _options$library = options.library,
      library = _options$library === undefined ? 'styled-components' : _options$library;


  var template = templates[library] || templates['styled-components'];

  return config.map(function (conf) {
    var parsed = parse(conf);
    var code = isComposite(parsed) ? composite(parsed) : template(parsed);
    var module = format(code, {
      semi: false,
      singleQuote: true
    });

    return {
      name: conf.name,
      module: module
    };
  });
};

var isComposite = function isComposite(comp) {
  return comp.imports && comp.jsx;
};