const { format } = require('prettier')
const templates = require('./templates')
const composite = require('./templates/composite')
const {
  stringifyObject,
  getExtensionImport,
  getComp,
  getSystemFuncs
} = require('./templates/util')

const parse = conf =>
  Object.assign({}, conf, {
    systemFunctions: getSystemFuncs(conf.system || []),
    extensionImport: getExtensionImport(conf.type),
    type: getComp(conf.type || 'div'),
    style: stringifyObject(conf.style || {}),
    props: stringifyObject(conf.props || {})
  })

module.exports = (config, options = {}) => {
  const { library = 'styled-components' } = options

  const template = templates[library] || templates['styled-components']

  return config.map(conf => {
    const parsed = parse(conf)
    const code = isComposite(parsed) ? composite(parsed) : template(parsed)
    const module = format(code, {
      semi: false,
      singleQuote: true
    })

    return {
      name: conf.name,
      module
    }
  })
}

const isComposite = comp => comp.imports && comp.jsx
